package com.example.backend.books.service;

import java.util.List;
import java.util.stream.StreamSupport;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.example.backend.books.model.BookEntity;
import com.example.backend.books.repository.BookRepository;
import com.example.backend.core.error.NotFoundException;

@Service
public class BookService {
    private final BookRepository repository;

    public BookService(BookRepository repository) {
        this.repository = repository;
    }

    @Transactional(readOnly = true)
    public List<BookEntity> getAll(long categoryId) {
        if (categoryId <= 0L) {
            return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
        }
        return repository.findByCategoryId(categoryId);
    }

    @Transactional(readOnly = true)
    public Page<BookEntity> getAll(long categoryId, int page, int size) {
        final Pageable pageRequest = PageRequest.of(page, size);
        if (categoryId <= 0L) {
            return repository.findAll(pageRequest);
        }
        return repository.findByCategoryId(categoryId, pageRequest);
    }

    @Transactional(readOnly = true)
    public BookEntity get(long id) {
        return repository.findById(id)
                .orElseThrow(() -> new NotFoundException(BookEntity.class, id));
    }

    @Transactional
    public BookEntity create(BookEntity entity) {
        if (entity == null) {
            throw new IllegalArgumentException("Entity is null");
        }
        return repository.save(entity);
    }

    @Transactional
    public BookEntity update(Long id, BookEntity entity) {
        final BookEntity existsEntity = get(id);
        existsEntity.setCategory(entity.getCategory());
        existsEntity.setTitle(entity.getTitle());
        existsEntity.setAuthor(entity.getAuthor());
        existsEntity.setPrice(entity.getPrice());
        existsEntity.setCount(entity.getCount());
        existsEntity.setDescription(entity.getDescription());
        existsEntity.setAnnotation(entity.getAnnotation());
        existsEntity.setBookCode(entity.getBookCode());
        existsEntity.setPublisher(entity.getPublisher());
        existsEntity.setSeries(entity.getSeries());
        existsEntity.setPublicationYear(entity.getPublicationYear());
        existsEntity.setPagesNum(entity.getPagesNum());
        existsEntity.setSize(entity.getSize());
        existsEntity.setCoverType(entity.getCoverType());
        existsEntity.setCirculation(entity.getCirculation());
        existsEntity.setWeight(entity.getWeight());
        existsEntity.setImage(entity.getImage());
        return repository.save(existsEntity);
    }

    @Transactional
    public BookEntity delete(Long id) {
        final BookEntity existsEntity = get(id);
        repository.delete(existsEntity);
        return existsEntity;
    }
}
