package com.example.backend.categories.api;

import java.util.List;

import org.modelmapper.ModelMapper;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.example.backend.categories.model.CategoryEntity;
import com.example.backend.categories.service.CategoryService;
import com.example.backend.core.configuration.Constants;

import jakarta.validation.Valid;

@RestController
@RequestMapping(Constants.API_URL + "/category")
public class CategoryController {
    private final CategoryService categoryService;
    private final ModelMapper modelMapper;

    public CategoryController(CategoryService categoryService, ModelMapper modelMapper) {
        this.categoryService = categoryService;
        this.modelMapper = modelMapper;
    }

    private CategoryDto toDto(CategoryEntity entity) {
        return modelMapper.map(entity, CategoryDto.class);
    }

    private CategoryEntity toEntity(CategoryDto dto) {
        return modelMapper.map(dto, CategoryEntity.class);
    }

    @GetMapping
    public List<CategoryDto> getAll() {
        return categoryService.getAll().stream().map(this::toDto).toList();
    }

    @GetMapping("/{id}")
    public CategoryDto get(@PathVariable(name = "id") Long id) {
        return toDto(categoryService.get(id));
    }

    @PostMapping
    public CategoryDto create(@RequestBody @Valid CategoryDto dto) {
        return toDto(categoryService.create(toEntity(dto)));
    }

    @PutMapping("/{id}")
    public CategoryDto update(@PathVariable(name = "id") Long id, @RequestBody @Valid CategoryDto dto) {
        return toDto(categoryService.update(id, toEntity(dto)));
    }

    @DeleteMapping("/{id}")
    public CategoryDto delete(@PathVariable(name = "id") Long id) {
        return toDto(categoryService.delete(id));
    }
}
